@@ -300,7 +300,7 @@ def handle_edit(cli_flags: Namespace):
     loop = asyncio.new_event_loop()
     asyncio.set_event_loop(loop)
     data_manager.load_basic_configuration(cli_flags.instance_name)
-    red = Red(cli_flags=cli_flags, description="Red V3", dm_help=None, fetch_offline_members=True)
+    red = Red(cli_flags=cli_flags, description="Red V3", dm_help=None)
     try:
         driver_cls = drivers.get_driver_class()
         loop.run_until_complete(driver_cls.initialize(**data_manager.storage_details()))
@@ -319,7 +319,7 @@ def handle_edit(cli_flags: Namespace):
 async def run_bot(red: Red, cli_flags: Namespace) -> None:
     """
     This runs the bot.
-    
+
     Any shutdown which is a result of not being able to log in needs to raise
     a SystemExit exception.
 
@@ -391,6 +391,13 @@ async def run_bot(red: Red, cli_flags: Namespace) -> None:
                 print("Token has been reset.")
                 sys.exit(0)
         sys.exit(1)
+    except discord.PrivilegedIntentsRequired:
+        print(
+            "Red requires all Privileged Intents to be enabled.\n"
+            "You can find out how to enable Privileged Intents with this guide:\n"
+            "https://docs.discord.red/en/stable/bot_application_guide.html#enabling-privileged-intents"
+        )
+        sys.exit(1)
 
     return None
 
@@ -487,9 +494,7 @@ def main():
 
         data_manager.load_basic_configuration(cli_flags.instance_name)
 
-        red = Red(
-            cli_flags=cli_flags, description="Red V3", dm_help=None, fetch_offline_members=True
-        )
+        red = Red(cli_flags=cli_flags, description="Red V3", dm_help=None)
 
         if os.name != "nt":
             # None of this works on windows.
@@ -503,7 +508,7 @@ def main():
         exc_handler = functools.partial(global_exception_handler, red)
         loop.set_exception_handler(exc_handler)
         # We actually can't (just) use asyncio.run here
-        # We probably could if we didnt support windows, but we might run into
+        # We probably could if we didn't support windows, but we might run into
         # a scenario where this isn't true if anyone works on RPC more in the future
         fut = loop.create_task(run_bot(red, cli_flags))
         r_exc_handler = functools.partial(red_exception_handler, red)
@@ -528,7 +533,7 @@ def main():
             loop.run_until_complete(shutdown_handler(red, None, ExitCodes.CRITICAL))
     finally:
         # Allows transports to close properly, and prevent new ones from being opened.
-        # Transports may still not be closed correcly on windows, see below
+        # Transports may still not be closed correctly on windows, see below
         loop.run_until_complete(loop.shutdown_asyncgens())
         # *we* aren't cleaning up more here, but it prevents
         # a runtime error at the event loop on windows
